@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2007 by Tobias Koenig <tokoe@kde.org>                   *
  *   Copyright (C) 2008 by Marco Martin <notmart@gmail.com>                *
- *   Copyright (C) 2008-2010 Matthias Fuchs <mat69@gmx.net>                *
+ *   Copyright (C) 2008-2011 Matthias Fuchs <mat69@gmx.net>                *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -46,9 +46,13 @@
 #include <Nepomuk/ResourceManager>
 #include <Nepomuk/Tag>
 #include <Nepomuk/Variant>
+#include <Nepomuk/Vocabulary/NCO>
+#include <Nepomuk/Vocabulary/NIE>
+#include <Nepomuk/Vocabulary/NFO>
+#include <Nepomuk/Vocabulary/PIMO>
+#include <nepomuk/utils.h>
 
-#include <Soprano/Vocabulary/NAO>
-#include <Soprano/Vocabulary/Xesam>
+using namespace Nepomuk::Vocabulary;
 #endif
 
 #include <Plasma/Theme>
@@ -325,16 +329,26 @@ void ComicApplet::setTabBarVisible( bool isVisible )
     }
 }
 
-void ComicApplet::dataUpdated( const QString&, const Plasma::DataEngine::Data &data )
+void ComicApplet::dataUpdated( const QString &source, const Plasma::DataEngine::Data &data )
 {
+    //disconnect prefetched comic strips
+    if ( source != mOldSource ) {
+        mEngine->disconnectSource( source, this );
+        return;
+    }
+
     setBusy( false );
     setConfigurationRequired( false );
     slotStartTimer();
 
     //there was an error, display information as image
-    if ( data[ "Error" ].toBool() ) {
-        if ( !mShowErrorPicture && !data[ "Previous identifier suffix" ].toString().isEmpty() ) {
-            updateComic( data[ "Previous identifier suffix" ].toString() );
+    const bool hasError = data[ "Error" ].toBool();
+    const bool errorAutoFixable = data[ "Error automatically fixable" ].toBool();
+    if ( hasError ) {
+        const QString previousIdentifierSuffix = data[ "Previous identifier suffix" ].toString();
+        if ( !mShowErrorPicture && !previousIdentifierSuffix.isEmpty() ) {
+            mEngine->disconnectSource( source, this );
+            updateComic( previousIdentifierSuffix );
             return;
         }
         QPixmap errorPic( 500, 400 );
@@ -372,6 +386,7 @@ void ComicApplet::dataUpdated( const QString&, const Plasma::DataEngine::Data &d
 
     mWebsiteUrl = data[ "Website Url" ].value<KUrl>();
     setAssociatedApplicationUrls(mWebsiteUrl);
+    mImageUrl = data["Image Url"].value<KUrl>();
     mShopUrl = data[ "Shop Url" ].value<KUrl>();
     mFirstIdentifierSuffix = data[ "First strip identifier suffix" ].toString();
     mStripTitle = data[ "Strip title" ].toString();
@@ -420,6 +435,11 @@ void ComicApplet::dataUpdated( const QString&, const Plasma::DataEngine::Data &d
     mLabelUrl->setText( mWebsiteUrl.host() );
     mImageWidget->setScaled( !mScaleComic );
 
+    //disconnect if there is either no error, or an error that can not be fixed automatically 
+    if ( !errorAutoFixable ) {
+        mEngine->disconnectSource( source, this );
+    }
+
     setTabBarVisible( mShowTabBar && mUseTabs );
     mLabelTop->setVisible( ( mShowComicAuthor || mShowComicTitle ) && !mLabelTop->text().isEmpty() );
     mLabelId->setVisible( mShowComicIdentifier && !mLabelId->text().isEmpty() );
@@ -449,11 +469,14 @@ void ComicApplet::dataUpdated( const QString&, const Plasma::DataEngine::Data &d
 
     //prefetch the previous and following comic for faster navigation
     if ( !mNextIdentifierSuffix.isEmpty() ) {
-        mEngine->query( mComicIdentifier + ':' + mNextIdentifierSuffix );
+        const QString prefetch = mComicIdentifier + ':' + mNextIdentifierSuffix;
+        mEngine->connectSource( prefetch, this );
+        mEngine->query( prefetch );
     }
-
     if ( !mPreviousIdentifierSuffix.isEmpty() ) {
-        mEngine->query( mComicIdentifier + ':' + mPreviousIdentifierSuffix );
+        const QString prefetch = mComicIdentifier + ':' + mPreviousIdentifierSuffix;
+        mEngine->connectSource( prefetch, this );
+        mEngine->query( prefetch );
     }
 }
 
@@ -632,9 +655,6 @@ void ComicApplet::configChanged()
     mSwitchTabs = cg.readEntry( "switchTabs", false );
     mTabView = cg.readEntry( "tabView", ShowText | ShowIcon );
     mSavingDir = cg.readEntry( "savingDir", QString() );
-    mOldSource = mComicIdentifier + ':' + mStoredIdentifierSuffix;
-
-    
 }
 
 void ComicApplet::saveConfig()
@@ -833,6 +853,7 @@ void ComicApplet::updateComic( const QString &identifierSuffix )
 
     setConfigurationRequired( mComicIdentifier.isEmpty() );
     if ( !mComicIdentifier.isEmpty() && mEngine && mEngine->isValid() ) {
+
         setBusy( true );
         const QString identifier = mComicIdentifier + ':' + identifierSuffix;
 
@@ -928,26 +949,45 @@ void ComicApplet::slotSaveComicAs()
 #ifdef HAVE_NEPOMUK
     bool worked = KIO::NetAccess::file_copy( srcUrl, destUrl );
     //store additional data using Nepomuk
-    if ( worked ) {
-        Nepomuk::Resource res( destUrl );
-        Nepomuk::Tag tag( i18n( "Comic" ) );
+    if (worked) {
+        Nepomuk::Resource res(destUrl, NFO::FileDataObject());
 
-        tag.setLabel( i18n( "Comic" ) );
-        res.addTag( tag );
-        if ( !mAdditionalText.isEmpty() ) {
-            res.setDescription( mAdditionalText );
+        Nepomuk::Resource comicTopic("Comic", PIMO::Topic());
+        comicTopic.setLabel(i18n("Comic"));
+
+        if (!mAdditionalText.isEmpty() ) {
+            res.setProperty(NIE::description(), mAdditionalText);
         }
-        if ( !mComicAuthor.isEmpty() ) {
-            res.setProperty( Soprano::Vocabulary::NAO::creator(), Nepomuk::Variant( mComicAuthor ) );
+        if ((mSuffixType == "Date") && !mShownIdentifierSuffix.isEmpty()) {
+            res.setProperty(NIE::contentCreated(), QDateTime::fromString(mShownIdentifierSuffix, Qt::ISODate));
         }
-        if ( ( mSuffixType == "Date" ) && !mShownIdentifierSuffix.isEmpty() ) {
-            res.setProperty( Soprano::Vocabulary::NAO::created(), Nepomuk::Variant( QDate::fromString( mShownIdentifierSuffix, Qt::ISODate ) ) );
+        if (!mComicTitle.isEmpty()) {
+            Nepomuk::Resource topic(mComicTitle, PIMO::Topic());
+            topic.setLabel(mComicTitle);
+            topic.setProperty(PIMO::superTopic(), comicTopic);
+            res.addTag(topic);
+        } else {
+//             res.addTag(comicTopic);//TODO activate this, see below
+            ;
         }
-        if ( !mComicTitle.isEmpty() ) {
-            res.setProperty( Soprano::Vocabulary::Xesam::subject(), Nepomuk::Variant( mComicTitle ) );
+
+        //FIXME also set the comic topic as tag, this is redundant, as topic has this as super topic
+        //though at this point the gui does not manage to show the correct tags
+        res.addTag(comicTopic);
+
+        if (!mStripTitle.isEmpty()) {
+            res.setProperty(NIE::title(), mStripTitle);
         }
-        if ( !mStripTitle.isEmpty() ) {
-            res.setProperty( Soprano::Vocabulary::Xesam::title(), Nepomuk::Variant( mStripTitle ) );
+        if (!mWebsiteUrl.isEmpty()) {
+            Nepomuk::Resource copyEvent = Nepomuk::Utils::createCopyEvent(mImageUrl, destUrl, QDateTime(), mWebsiteUrl);
+        }
+
+        const QStringList authors = mComicAuthor.split(',', QString::SkipEmptyParts);
+        foreach (QString author, authors) {
+            author = author.trimmed();
+            Nepomuk::Resource authorRes(author, NCO::PersonContact());
+            authorRes.setProperty(NCO::fullname(), author);
+            res.addProperty(NCO::creator(), authorRes);
         }
     }
 #else
