@@ -45,6 +45,8 @@
 #include <Plasma/DataEngine>
 #include <Plasma/ToolButton>
 #include <Plasma/Frame>
+#include <Plasma/ToolTipContent>
+#include <Plasma/ToolTipManager>
 
 #include <math.h>
 
@@ -97,6 +99,36 @@ void Frame::init()
     m_currentDay = QDate::currentDate();
     
     m_slideNumber = 0;
+
+    // Frame & Shadow dimensions
+    m_frameOutline = 8;
+    m_swOutline = 8;
+
+    // Initialize the slideshow timer
+    connect(m_mySlideShow, SIGNAL(pictureUpdated()), this, SLOT(scalePictureAndUpdate()));
+
+    connect(&m_waitForResize, SIGNAL(timeout()), this, SLOT(scalePictureAndUpdate()));
+    m_waitForResize.setSingleShot(true);
+    m_waitForResize.setInterval(200);
+    
+    configChanged();
+    initSlideShow();
+    
+    KConfigGroup cg = config();
+    if (frameReceivedUrlArgs) {
+        cg.writeEntry("url", m_currentUrl);
+        emit configNeedsSaving();
+    }
+
+    m_menuPresent = false;
+
+    QAction *openAction = action("run associated application");
+    openAction->setIcon(SmallIcon("image-x-generic"));
+    openAction->setText(i18n("&Open Picture..."));    
+}
+
+void Frame::configChanged()
+{
     // Get config values
     KConfigGroup cg = config();
     m_frameColor = cg.readEntry("frameColor", QColor(70, 90, 130)); //theme?
@@ -113,29 +145,6 @@ void Frame::init()
     m_potdProvider = cg.readEntry("potdProvider", QString());
     m_potd = cg.readEntry("potd", false);
     m_autoUpdateIntervall = cg.readEntry("autoupdate time", 0);
-
-    // Frame & Shadow dimensions
-    m_frameOutline = 8;
-    m_swOutline = 8;
-
-    // Initialize the slideshow timer
-    connect(m_mySlideShow, SIGNAL(pictureUpdated()), this, SLOT(scalePictureAndUpdate()));
-
-    connect(&m_waitForResize, SIGNAL(timeout()), this, SLOT(scalePictureAndUpdate()));
-    m_waitForResize.setSingleShot(true);
-    m_waitForResize.setInterval(200);
-
-    initSlideShow();
-    if (frameReceivedUrlArgs) {
-        cg.writeEntry("url", m_currentUrl);
-        emit configNeedsSaving();
-    }
-
-    m_menuPresent = false;
-
-    QAction *openAction = action("run associated application");
-    openAction->setIcon(SmallIcon("image-x-generic"));
-    openAction->setText(i18n("&Open Picture..."));
 }
 
 void Frame::slotOpenPicture()
@@ -266,6 +275,9 @@ void Frame::updatePicture()
     }
 
     kDebug() << "Rendering picture";
+    Plasma::ToolTipContent toolTipData;
+    toolTipData.setSubText(m_mySlideShow->currentUrl().fileName());
+    Plasma::ToolTipManager::self()->setContent(this, toolTipData);
 
     // create a QPixmap which can be drawn in paintInterface()
     QPixmap picture = QPixmap::fromImage(m_scaledImage);
@@ -386,12 +398,15 @@ void Frame::updatePicture()
         // Set the font and draw text
         p->setRenderHint(QPainter::Antialiasing);
         QFont textFont = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-        textFont.setPixelSize(qMax(KGlobalSettings::smallestReadableFont().pixelSize(), bgRect.height() / 6));
+        textFont.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(), bgRect.height() / 6));
         p->setFont(textFont);
 
         QTextOption option;
         option.setAlignment(Qt::AlignCenter);
         option.setWrapMode(QTextOption::WordWrap);
+
+        preparePainter(p, bgRect, textFont, message);
+
         p->setPen(QPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor), 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
         p->drawText(bgRect, message, option);
     }
@@ -406,6 +421,33 @@ void Frame::updatePicture()
     }
 }
 
+QRect Frame::preparePainter(QPainter *p, const QRect &rect, const QFont &font, const QString &text)
+{
+    QRect tmpRect;
+    QFont tmpFont = font;
+    bool first = true;
+
+    // Starting with the given font, decrease its size until it'll fit in the
+    // given rect allowing wrapping where possible
+    do {
+        if (first) {
+            first = false;
+        } else  {
+            tmpFont.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(), tmpFont.pointSize() - 1));
+        }
+
+        const QFontMetrics fm(tmpFont);
+        int flags = Qt::TextWordWrap;
+
+        tmpRect = fm.boundingRect(rect, flags, text);
+    } while (tmpFont.pointSize() > KGlobalSettings::smallestReadableFont().pointSize() &&
+             (tmpRect.width() > rect.width() || tmpRect.height() > rect.height()));
+    
+    p->setFont(tmpFont);
+    return tmpRect;
+}
+
+
 void Frame::nextPicture()
 {
     m_mySlideShow->setUpdateInterval(0);
@@ -626,6 +668,9 @@ void Frame::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 
 void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
+    if (m_slideFrame) {
+        m_slideFrame->hide();
+    }
     KUrl droppedUrl = (KUrl::List::fromMimeData(event->mimeData())).at(0);
     kDebug() << "dropped URL" << droppedUrl.url();
     if (droppedUrl.protocol() == "desktop") {
@@ -657,8 +702,8 @@ void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
 
 void Frame::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
 {
+    checkSlideFrame();
     if (m_slideShow) {
-        checkSlideFrame();
         m_slideFrame->show();
     }
     Applet::hoverEnterEvent(event);
@@ -676,6 +721,7 @@ void Frame::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
 void Frame::checkSlideFrame()
 {
     if (m_slideFrame) {
+        m_slideFrame->hide();
         return;
     }
     m_slideFrame = new Plasma::Frame( this );
