@@ -26,11 +26,13 @@
 
 
 BackspaceKey::BackspaceKey(QPoint relativePosition, QSize relativeSize) :
-        FuncKey(relativePosition, relativeSize, Helpers::keysymToKeycode(XK_BackSpace), QString()){
+        FuncKey(relativePosition, relativeSize, Helpers::keysymToKeycode(XK_BackSpace), QString())
+{
 
 }
 
-void BackspaceKey::paint(QPainter *painter){
+void BackspaceKey::paint(QPainter *painter)
+{
     FuncKey::paint(painter);
 
     painter->save();
