@@ -19,6 +19,7 @@
 
 #include "weatherview.h"
 
+#include <QGraphicsSceneContextMenuEvent>
 #include <QTreeView>
 #include <QHeaderView>
 
@@ -33,6 +34,8 @@ namespace Plasma
 WeatherView::WeatherView(QGraphicsWidget *parent)
         : TreeView(parent)
 {
+    setAcceptedMouseButtons(Qt::NoButton);
+    
     QTreeView *native = nativeWidget();
     native->viewport()->setAutoFillBackground(false);
     native->header()->hide();
@@ -98,6 +101,11 @@ QSizeF WeatherView::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
     }
 }
 
+void WeatherView::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
+{
+    event->ignore();
+}
+
 }
 
 #include <weatherview.moc>
