@@ -14,16 +14,11 @@
 #include "virus.h"
 
 #include <QPainter>
-#include <QFile>
 
-#include <KDirSelectDialog>
+#include <KDebug>
 #include <KDirWatch>
 #include <KFileDialog>
-#include <KGlobalSettings>
-#include <KImageFilePreview>
 #include <KNS/Engine>
-#include <KProgressDialog>
-#include <KRandom>
 #include <KStandardDirs>
 
 #include <Plasma/Theme>
@@ -45,7 +40,7 @@ Virus::Virus(QObject *parent, const QVariantList &args)
 {
     connect(this, SIGNAL(renderCompleted(QImage)), this, SLOT(updateBackground(QImage)));
     connect(&alife, SIGNAL(finished()), this, SLOT(virusUpdated()));
-    
+
     //m_timer = QTimer(this);
     m_timer.setSingleShot(true);
     connect(&m_timer, SIGNAL(timeout()), this, SLOT(requestUpdate()));
@@ -60,14 +55,14 @@ Virus::~Virus()
 void Virus::init(const KConfigGroup &config)
 {
     m_timer.stop();
-    m_mode = "SingleImage";//renderingMode().name();
+    m_mode = QLatin1String( "SingleImage" );//renderingMode().name();
     calculateGeometry();
 
     m_resizeMethod = (ResizeMethod)config.readEntry("wallpaperposition", (int)ScaledResize);
     m_wallpaper = config.readEntry("wallpaper", QString());
     if (m_wallpaper.isEmpty()) {
         m_wallpaper = Plasma::Theme::defaultTheme()->wallpaperPath();
-        int index = m_wallpaper.indexOf("/contents/images/");
+        int index = m_wallpaper.indexOf(QLatin1String( "/contents/images/" ));
         if (index > -1) { // We have file from package -> get path to package
             m_wallpaper = m_wallpaper.left(index);
         }
@@ -97,7 +92,7 @@ void Virus::requestUpdate()
 void Virus::virusUpdated()
 {
     m_pixmap = QPixmap::fromImage(alife.currentImage());
-    emit update(boundingRect());
+    emit update(alife.updatedArea());
     m_timer.start(alife.getUpdateInterval());
 }
 
@@ -116,6 +111,7 @@ void Virus::save(KConfigGroup &config)
 void Virus::configWidgetDestroyed()
 {
     m_configWidget = 0;
+    m_model = 0;
 }
 
 QWidget* Virus::createConfigurationInterface(QWidget* parent)
@@ -126,29 +122,27 @@ QWidget* Virus::createConfigurationInterface(QWidget* parent)
     m_uiVirus.setupUi(m_configWidget);
 
     qreal ratio = m_size.isEmpty() ? 1.0 : m_size.width() / qreal(m_size.height());
-    m_model = new BackgroundListModel(ratio, this);
+    m_model = new BackgroundListModel(ratio, this, m_configWidget);
     m_model->setResizeMethod(m_resizeMethod);
     m_model->setWallpaperSize(m_size);
     m_model->reload(m_usersWallpapers);
     m_uiVirus.m_view->setModel(m_model);
-    m_uiVirus.m_view->setItemDelegate(new BackgroundDelegate(m_uiVirus.m_view->view(),
-							      ratio, this));
-    m_uiVirus.m_view->view()->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
-    int index = m_model->indexOf(m_wallpaper);
-    if (index != -1) {
-	m_uiVirus.m_view->setCurrentIndex(index);
-	Plasma::Package *b = m_model->package(index);
-	if (b) {
-	    fillMetaInfo(b);
-	}
+    m_uiVirus.m_view->setItemDelegate(new BackgroundDelegate(m_uiVirus.m_view,
+                                                                 ratio, m_configWidget));
+    m_uiVirus.m_view->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
+    QModelIndex index = m_model->indexOf(m_wallpaper);
+    if (index.isValid()) {
+        m_uiVirus.m_view->setCurrentIndex(index);
+        Plasma::Package *b = m_model->package(index.row());
+        if (b) {
+            fillMetaInfo(b);
+        }
     }
-    connect(m_uiVirus.m_view, SIGNAL(currentIndexChanged(int)), this, SLOT(pictureChanged(int)));
+    connect(m_uiVirus.m_view, SIGNAL(activated(const QModelIndex &)), this, SLOT(pictureChanged(const QModelIndex &)));
 
-    m_uiVirus.m_pictureUrlButton->setIcon(KIcon("document-open"));
+    m_uiVirus.m_pictureUrlButton->setIcon(KIcon(QLatin1String( "document-open" )));
     connect(m_uiVirus.m_pictureUrlButton, SIGNAL(clicked()), this, SLOT(showFileDialog()));
 
-    m_uiVirus.m_emailLine->setTextInteractionFlags(Qt::TextSelectableByMouse);
-
     m_uiVirus.m_resizeMethod->addItem(i18n("Scaled & Cropped"), ScaledAndCroppedResize);
     m_uiVirus.m_resizeMethod->addItem(i18n("Scaled"), ScaledResize);
     m_uiVirus.m_resizeMethod->addItem(i18n("Scaled, keep proportions"), MaxpectResize);
@@ -156,13 +150,14 @@ QWidget* Virus::createConfigurationInterface(QWidget* parent)
     m_uiVirus.m_resizeMethod->addItem(i18n("Tiled"), TiledResize);
     m_uiVirus.m_resizeMethod->addItem(i18n("Center Tiled"), CenterTiledResize);
     for (int i = 0; i < m_uiVirus.m_resizeMethod->count(); ++i) {
-	if (m_resizeMethod == m_uiVirus.m_resizeMethod->itemData(i).value<int>()) {
-	    m_uiVirus.m_resizeMethod->setCurrentIndex(i);
-	    break;
-	}
+        if (m_resizeMethod == m_uiVirus.m_resizeMethod->itemData(i).value<int>()) {
+            m_uiVirus.m_resizeMethod->setCurrentIndex(i);
+            break;
+        }
     }
     connect(m_uiVirus.m_resizeMethod, SIGNAL(currentIndexChanged(int)),
-	    this, SLOT(positioningChanged(int)));
+            this, SLOT(positioningChanged(int)));
+    m_uiVirus.m_newStuff->setIcon(KIcon(QLatin1String( "get-hot-new-stuff" )));
 
     m_uiVirus.m_color->setColor(m_color);
     connect(m_uiVirus.m_color, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
@@ -177,12 +172,19 @@ QWidget* Virus::createConfigurationInterface(QWidget* parent)
     connect(m_uiVirus.m_showCells, SIGNAL(stateChanged(int)), this, SLOT(showCellsChanged(int)));
 
     connect(m_uiVirus.m_newStuff, SIGNAL(clicked()), this, SLOT(getNewWallpaper()));
-    
+
     connect(this, SIGNAL(settingsChanged(bool)), parent, SLOT(settingsChanged(bool)));
-    
+
+    connect(m_uiVirus.m_view, SIGNAL(clicked(const QModelIndex &)), this, SLOT(modified()));
+
     return m_configWidget;
 }
 
+void Virus::modified()
+{
+    emit settingsChanged(true);
+}
+
 void Virus::calculateGeometry()
 {
     m_size = boundingRect().size().toSize();
@@ -231,14 +233,29 @@ void Virus::setSingleImage()
     }
 
     QString img;
-    Plasma::Package b(m_wallpaper, packageStructure(this));
 
-    img = b.filePath("preferred");
-    kDebug() << img << m_wallpaper;
-    if (img.isEmpty()) {
-        img = m_wallpaper;
+    if (QDir::isAbsolutePath(m_wallpaper)) {
+        Plasma::Package b(m_wallpaper, packageStructure(this));
+        img = b.filePath("preferred");
+        kDebug() << img << m_wallpaper;
+
+        if (img.isEmpty()) {
+            img = m_wallpaper;
+        }
+    //if it's not an absolute path, check if it's just a wallpaper name
+    } else {
+        const QString path = KStandardDirs::locate("wallpaper", m_wallpaper + QLatin1String( "/metadata.desktop" ));
+
+        if (!path.isEmpty()) {
+            QDir dir(path);
+            dir.cdUp();
+
+            Plasma::Package b(dir.path(), packageStructure(this));
+            img = b.filePath("preferred");
+        }
     }
 
+
     if (!m_size.isEmpty()) {
         renderWallpaper(img);
     }
@@ -247,7 +264,7 @@ void Virus::setSingleImage()
 void Virus::getNewWallpaper()
 {
     KNS::Engine engine(m_configWidget);
-    if (engine.init("virus_wallpaper.knsrc")) {
+    if (engine.init(QLatin1String( "virus_wallpaper.knsrc" ))) {
         KNS::Entry::List entries = engine.downloadDialogModal(m_configWidget);
 
         if (entries.size() > 0 && m_model) {
@@ -263,13 +280,13 @@ void Virus::colorChanged(const QColor& color)
     emit settingsChanged(true);
 }
 
-void Virus::pictureChanged(int index)
+void Virus::pictureChanged(QModelIndex index)
 {
-    if (index == -1 || !m_model) {
+    if (index.row() == -1 || !m_model) {
         return;
     }
 
-    Plasma::Package *b = m_model->package(index);
+    Plasma::Package *b = m_model->package(index.row());
     if (!b) {
         return;
     }
@@ -319,33 +336,11 @@ void Virus::positioningChanged(int index)
 
 void Virus::fillMetaInfo(Plasma::Package *b)
 {
-    // Prepare more user-friendly forms of some pieces of data.
-    // - license by config is more a of a key value,
-    //   try to get the proper name if one of known licenses.
-
-    // not needed for now...
-    //QString license = b->license();
-    /*
-    KAboutLicense knownLicense = KAboutLicense::byKeyword(license);
-    if (knownLicense.key() != KAboutData::License_Custom) {
-        license = knownLicense.name(KAboutData::ShortName);
-    }
-    */
-    // - last ditch attempt to localize author's name, if not such by config
-    //   (translators can "hook" names from outside if resolute enough).
-    QString author = b->metadata().author();
-    if (author.isEmpty()) {
-        setMetadata(m_uiVirus.m_authorLine, QString());
-        m_uiVirus.m_authorLabel->setAlignment(Qt::AlignLeft);
-    } else {
-        QString authorIntl = i18nc("Wallpaper info, author name", "%1", author);
-        m_uiVirus.m_authorLabel->setAlignment(Qt::AlignRight);
-        setMetadata(m_uiVirus.m_authorLine, authorIntl);
-    }
-    setMetadata(m_uiVirus.m_licenseLine, QString());
+  QString author = b->metadata().author();
+    /*setMetadata(m_uiVirus.m_licenseLine, QString());
     setMetadata(m_uiVirus.m_emailLine, QString());
     m_uiVirus.m_emailLabel->hide();
-    m_uiVirus.m_licenseLabel->hide();
+    m_uiVirus.m_licenseLabel->hide();  */
 }
 
 bool Virus::setMetadata(QLabel *label, const QString &text)
@@ -364,11 +359,14 @@ bool Virus::setMetadata(QLabel *label, const QString &text)
 void Virus::showFileDialog()
 {
     if (!m_dialog) {
-        m_dialog = new KFileDialog(KUrl(), "*.png *.jpeg *.jpg *.xcf *.svg *.svgz", m_configWidget);
+        m_dialog = new KFileDialog(KUrl(), QLatin1String( "*.png *.jpeg *.jpg *.xcf *.svg *.svgz *.bmp" ), m_configWidget);
         m_dialog->setOperationMode(KFileDialog::Opening);
         m_dialog->setInlinePreviewShown(true);
         m_dialog->setCaption(i18n("Select Wallpaper Image File"));
         m_dialog->setModal(false);
+
+        connect(m_dialog, SIGNAL(okClicked()), this, SLOT(wallpaperBrowseCompleted()));
+        connect(m_dialog, SIGNAL(destroyed(QObject*)), this, SLOT(fileDialogFinished()));
     }
 
     m_dialog->show();
@@ -382,8 +380,7 @@ void Virus::browse()
 {
     Q_ASSERT(m_model);
 
-    QString wallpaper = m_dialog->selectedFile();
-    disconnect(m_dialog, SIGNAL(okClicked()), this, SLOT(browse()));
+    const QString wallpaper = m_dialog->selectedFile();
 
     if (wallpaper.isEmpty()) {
         return;
@@ -398,8 +395,8 @@ void Virus::browse()
     m_model->addBackground(wallpaper);
 
     // select it
-    int index = m_model->indexOf(wallpaper);
-    if (index != -1) {
+    QModelIndex index = m_model->indexOf(wallpaper);
+    if (index.isValid()) {
         m_uiVirus.m_view->setCurrentIndex(index);
     }
     // save it
@@ -423,7 +420,7 @@ void Virus::renderWallpaper(const QString& image)
 QString Virus::cacheId() const
 {
     QSize s = boundingRect().size().toSize();
-    return QString("%5_%3_%4_%1x%2").arg(s.width()).arg(s.height()).arg(m_color.name()).arg(m_resizeMethod).arg(m_img);
+    return QString(QLatin1String( "%5_%3_%4_%1x%2" )).arg(s.width()).arg(s.height()).arg(m_color.name()).arg(m_resizeMethod).arg(m_img);
 }
 
 void Virus::updateBackground(const QImage &img)
@@ -453,7 +450,7 @@ void Virus::suspendStartup(bool suspend)
 
 void Virus::updateScreenshot(QPersistentModelIndex index)
 {
-    m_uiVirus.m_view->view()->update(index);
+    m_uiVirus.m_view->update(index);
 }
 
 #include "virus.moc"
